/* 
 * enCore
 * Eneso Tecnología de Adaptación, S.L. 
 */

#include <glib/gi18n.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include "layout.h"
#include "misc.h"
#include "protocol.h"
#include "configuration.h"

gint	current_language;
gchar*	language_name [N_LANGUAGES] =
{ "en", "es", "it", "el" };

gboolean show_dev_options	= FALSE;

static void
get_system_language		(void);
static void
get_system_keyb_layout	(void);


	void
load_config_file (void)
{
	GKeyFile*	file;
	GError*		error;
	gchar*		value_str;
	gboolean	value_bool;
	gchar*		locale_dir;
	gchar*		config_file;
	gboolean	file_ok;
	gint		i;

	/* Open the config file */
	file = g_key_file_new ();

	config_file = get_config_file ("/config");

	error = NULL;
	file_ok = g_key_file_load_from_file (file, config_file,
			G_KEY_FILE_NONE, &error);

	g_free (config_file);

	if (file_ok)
	{
		/* Read the selected language */
		error = NULL;
		value_str = g_key_file_get_string (file, "language", "lang", &error);

		if (error == NULL && strcmp (value_str, ""))
		{
			for (i = 0; i < N_LANGUAGES; i++)
			{
				if (!strcmp (value_str, language_name [i]))
				{
					current_language = i;
					break;
				}
			}
		}
		else 
		{
			get_system_language ();
			file_ok = FALSE;
		}

		/* Read the selected keyboard layout */
		error = NULL;
		value_str = g_key_file_get_string (file, "keyboard", "layout", &error);

		if (error == NULL && strcmp (value_str, ""))
		{
			for (i = 0; i < N_KEYB_LAYOUTS; i++)
			{
				if (!strcmp (value_str, keyb_layout_name [i]))
				{
					current_keyb_layout = i;
					break;
				}
			}
		}
		else 
		{
			get_system_keyb_layout ();
			file_ok = FALSE;
		}

		/* Other options */
		error = NULL;
		value_bool = g_key_file_get_boolean (file, "dev_options", "show_dev_options", &error);

		if (error == NULL)
			show_dev_options = value_bool;
	}
	else 
	{
		g_message ("Config file: %s", error->message);
		get_system_language ();
		get_system_keyb_layout ();
	}

	/* Gettext */
	locale_dir = get_data_file ("/locale/");
	if (!g_file_test (locale_dir, G_FILE_TEST_EXISTS))
	{
		g_free (locale_dir);
		locale_dir = g_strdup ("../po/locale/");
	}

	bindtextdomain ("encore", locale_dir);
	g_free (locale_dir);

	bind_textdomain_codeset ("encore", "UTF-8");
	textdomain ("encore");

	set_env_var ("LANG", language_name [current_language]);
	set_env_var ("LANGUAGE", language_name [current_language]);
	setlocale (LC_MESSAGES, language_name [current_language]);

	/* If there was an error, save a default config */
	if (!file_ok)
		update_config_file ();

	g_key_file_free (file);
}

	void
update_config_file (void)
{
	GKeyFile*	file;
	gchar*		data;
	gchar*		config_file;
	FILE*		f;

	/* Create the file */
	file = g_key_file_new ();

	/* Set the keys */
	g_key_file_set_string (file, "language", "lang", language_name [current_language]);
	g_key_file_set_string (file, "keyboard", "layout", keyb_layout_name [current_keyb_layout]);
	g_key_file_set_boolean (file, "dev_options", "show_dev_options", show_dev_options);

	/* And save it */
	data = g_key_file_to_data (file, NULL, NULL);

	config_file = get_config_file ("/config");

	f = (FILE*) g_fopen (config_file, "w");
	if (f)
	{
		fputs (data, f);
		fclose (f);
	}

	g_free (config_file);

	g_free (data);

	g_key_file_free (file);
}

	static void
get_system_language (void)
{
	const gchar* const* languages;
	gboolean	found;
	gint		i, j;

	/* Check whether one the system's locales is supported */
	languages = g_get_language_names ();

	found = FALSE;
	for (i = 0; languages [i] != NULL; i++)
	{
		for (j = 0; j < N_LANGUAGES; j++)
		{
			if (!g_ascii_strcasecmp (languages [i], language_name [j]))
			{
				current_language = j;
				found = TRUE;
				break;
			}
		}

		if (found)
			break;
	}

	/* Otherwise, use English */
	if (!found)
		current_language = 0;
}

	static void
get_system_keyb_layout (void)
{
	const gchar* const* languages;
	gboolean	found;
	gint		i, j;

	/* Check whether one the system's locales is supported */
	languages = g_get_language_names ();

	found = FALSE;
	for (i = 0; languages [i] != NULL; i++)
	{
		for (j = 0; j < N_KEYB_LAYOUTS; j++)
		{
			if (!g_ascii_strcasecmp (languages [i], keyb_layout_name [j]))
			{
				current_keyb_layout = j;
				found = TRUE;
				break;
			}
		}

		if (found)
			break;
	}

	/* Otherwise, use en_US */
	if (!found)
		current_keyb_layout = 0;
}
